﻿import event.*;

/**
 * 管理被註冊到接收事件的物件名單（亦即，實作EventListener
 * 的一個類別）。這個類別被一個事件來源用來儲存它的偵聽器。 
 */
class event.EventListenerList {
  // 偵聽器物件
  private var listeners:Array;

  /**
   * 建構子
   */
  public function EventListenerList (  ) {
    // 建立用來儲存偵聽器的陣列。
    listeners = new Array(  );
  }

  /**
   * 在清單中加入一個偵聽器
   *
   * @param   l   要被加入的偵聽器。必須實作EventListener。
   */
  public function addObj (l:EventListener):Boolean {
    // 找尋指定的偵聽器
    var len:Number = listeners.length;
    for (var i:Number = len; --i >= 0; ) {
      if (listeners[i] == l) {
        return false;
      }
    }

    // 新的偵聽器並不在清單中，所以可以將它加進去。
    listeners.push(l);
    return true;
  }

  /**
   * 從清單中移除一個偵聽器
   *
   * @param   l   要被移除的偵聽器。必須實作EventListener。
   */
  public function removeObj (l:EventListener):Boolean {
    // 找尋指定的偵聽器 
    var len:Number = listeners.length;
    for (var i = len; --i >= 0; ) {
      if (listeners[i] == l) {
        // 找到了，將它移除。
        listeners.splice(i, 1);
        // 不用再找了
        return true;
      }
    }
    return false;
  }

  /**
   * 傳回將被用在事件通知期間的完整偵聽器名單。
   */
  public function getListeners (  ):Array {
    // 傳回這個清單的一個副本，而不是清單本身。
    return listeners.slice(0);
  }
}