﻿import event.*;
import util.*;

/**
 * 產生隨機事件
 */
class util.Randomizer {
  // 將要接收事件的事件偵聽器
  private var listenerList:EventListenerList;
  // 檢查隨機事件的時間間隔毫秒 
  private var randInterval:Number;
  // 上次發生隨機事件的時間 
  private var lastEventTime:Date;

  /**
   * 建構子
   *
   * @param  interval  	等待檢查隨機事件的間隔毫秒值 
   * 
   * @param  odds 		每次檢查時，可能觸發事件的機率。
   *               		每個事件都有一個1/odds的機率被觸發。 
   */
  public function Randomizer (interval:Number, odds:Number) {
    // 建立EventListenerList來管理偵聽器。
    listenerList = new EventListenerList(  );

    // 初始化最新事件的時間
    lastEventTime = new Date(  );

    // 開始檢查事件
    start(interval, odds);
  }

  /**
   * 註冊接收隨機事件的物件
   */
  public function addRandomizerListener (l:RandomizerListener):Boolean {
    return listenerList.addObj(l);
  }
  
  /**
   * 註銷事件偵聽器物件
   */
  public function removeRandomizerListener (l:RandomizerListener):Boolean {
    return listenerList.removeObj(l);
  }

  /**
   *  開始執行迴圈來檢查隨機事件
   *
   * @param  interval	等待檢查隨機事件的間隔毫秒值。 
   *                   
   * @param  odds 		每次檢查時，可能觸發事件的機率。
   * 
   */
  public function start (interval:Number, odds:Number):Boolean {
    if (interval > 0 && odds > 1) {
      // 每隔interval指定的毫秒，呼叫this.check(  ) 
      randInterval = setInterval(this, "check", interval, odds);
      return true;
    }
    return false;
  }

  /**
   * 停止檢查隨機事件
   */
  public function stop (  ):Void {
    clearInterval(randInterval);
  }

  /**
   * 重新啟動事件檢查迴圈，可能伴隨新的odds。
   *
   * @param  interval 	等待檢查隨機事件的間隔毫秒值。 
   *
   * @param  odds 	每次檢查時，可能觸發事件的機率。
   * 
   */
  public function restart (interval:Number, odds:Number):Void {
    stop(  );
    start(interval, odds);
  } 

  /**
   * 根據目前的odds，檢查是否有產生隨機的事件。 
   *
   * @param  odds		每次檢查時，可能觸發事件的機率。
   */
  private function check (odds:Number):Void {
    // 區域變數
    var rand:Number = Math.floor(Math.random(  ) * odds);
    var now:Date = new Date(  );
    var elapsed:Number;

    // 如果隨機事件發生了…
    if (rand == 0) {
      // 計算自從上次事件發生以來所經過的時間。
      elapsed = now.getTime() - lastEventTime.getTime(  );
      lastEventTime = now;

      // 觸發事件
      fireOnRandomAction(elapsed);
    }
  }

  /**
   * 在所有的偵聽器上呼叫onRandomAction( )
   *
   * @param  elapsed   自從上次事件通知後所經過的時間量
   */
  private function fireOnRandomAction (elapsed:Number):Void {
    // 建立描述此事件的物件 
    var e:RandomizerEvent = new RandomizerEvent(this, elapsed);
    // 取得目前的事件偵聽器的名單
    var listeners:Array = listenerList.getListeners(  );

    // 向所有的事件偵聽器廣播事件
    for (var i:Number = 0; i < listeners.length; i++) {
      // 請注意，我們並沒有在此轉換成RandomizerListener類型。 
      // 關於為何不需要轉換類型的說明，請參閱 
      // 第三章的「陣列元素和類型檢驗」一節。
      listeners[i].onRandomAction(e);
    }
  }
}